// Description:  Serial management Interface protocol
//
// Revision History :
// --------------------------------------------------------------------
//   Ver  :| Author            :| Mod. Date :| Changes Made:
//   V1.0 :| Shih-An Li        :| 10/12/2016:| Initial Revision
// --------------------------------------------------------------------
module SMI_top (
                // clock reset interface 
                    iClk_50M,   // 50Mhz clock
                    iRst_n,     // reset negativite
                
                // SDI interface
                    iMD_Reg_Addr,  // register address
                    iMD_TXData,    // transmit data
                    iMD_CMD_Trig,  // command trigger valid
                    oMD_RXData,    // receive data
                    oMD_Finish,    // trigger, when send or receive a rxdata finish
                    iMD_RWn,
                    oMD_Busy,      // busy to MD operation
 
                // dp83848 rmii signal
                    oMDC,       // management data clock
                    MDIO        // management data I/O
                );

//===========================================================================
// PARAMETER declarations
//===========================================================================
parameter CLKFREQ= 50000000;  // 50M
parameter PHY_Address = 5'h07;  // RX0 and RX1 connect
//===========================================================================
// PORT declarations
//===========================================================================
                // clock reset interface
input               iClk_50M;       // 50Mhz clock
input               iRst_n;     // reset negativite
                
                // SDI interface
input               iMD_CMD_Trig;    // when iMD_Reg_Addr and TXdata/RXdata ready, send a triger to active transmit
input   [4:0]       iMD_Reg_Addr;  // Register address                    
input   [15:0]      iMD_TXData;    // transmit data
output  [15:0]      oMD_RXData;    // receive data
output              oMD_Finish;
input               iMD_RWn;      //  1: read, 0:write
output              oMD_Busy;      // busy 

                // dp83848 rmii signal
output              oMDC;       // management data clock
inout               MDIO;       // management data I/O

reg [3:0]   rCnt;
///////////////////////////////////////////////////////////////////
//=============================================================================
// REG/WIRE declarations
//=============================================================================
wire        wMDC_Clk;
reg [4:0]   rREG_Addr;
wire[4:0]   wPHY_Addr;
reg [15:0]  rTXData_Reg, rTXData_TempReg;
reg [15:0]  rRXData_Reg, rRXData_TempReg;
reg [7:0]   rState;
reg         rCMD_Trig_Dly;
reg         rRWn, rStart;
reg         rSDAT;
//=============================================================================
// Structural coding
//=============================================================================
assign  MDIO = (rMDIO) ? 1'bz: rSDAT;
assign  oMDC = wMDC_Clk;
assign  oMD_Busy = rBusy;
assign  wPHY_Addr = PHY_Address;
assign  oMD_Finish = rFinish;
assign oMD_RXData = rRXData_Reg;

always@(posedge iClk_50M)   rCnt <= rCnt +1; 
assign wMDC_Clk = rCnt[0];  // 25Mhz for MDC

always@(posedge iClk_50M or negedge iRst_n)   begin
    if(!iRst_n) begin
        rCMD_Trig_Dly <= 0;
        rRWn <= 0;
        rREG_Addr <= 0;
        rStart <= 0;
    end
    else begin
        rCMD_Trig_Dly <= iMD_CMD_Trig;
        rRWn <= rRWn;
        rREG_Addr <= rREG_Addr;
        rStart <= rStart;
        rTXData_TempReg <= rTXData_TempReg;
        if(rBusy) begin
            rStart <= 0;
        end
        else if({rCMD_Trig_Dly, iMD_CMD_Trig}==2'b01) begin
            rStart <= 1;
            rREG_Addr <= iMD_Reg_Addr;
            rTXData_TempReg <= iMD_TXData;
            rRWn <= iMD_RWn;
        end
        else if( rStart ) begin // hold signal
            // hold signal
        end
        else  begin
            rStart <= 0;
        end
    end
end
reg     rMDIO, rBusy, rFinish, rStart_dly, rMD_RWn;
always@(negedge wMDC_Clk or negedge iRst_n)   begin
    if(!iRst_n) begin
        rMDIO = 0;
        rSDAT = 1;  // send 32 clk 1 to sync
        rBusy = 0;
        rFinish = 0;
        rState = 0;
        rStart_dly = 0;
        rMD_RWn = 0;
        rTXData_Reg = 0;
        rRXData_Reg = 0;
        rRXData_TempReg = 0;

    end
    else begin
        rMDIO = rMDIO;
        rBusy = rBusy;
        rFinish = rFinish;
        rStart_dly <= rStart;
        rMD_RWn = rMD_RWn;
        rState = rState;
        rTXData_Reg = rTXData_Reg;
        rRXData_Reg = rRXData_Reg;
        rRXData_TempReg = rRXData_TempReg;
        rSDAT = rSDAT;
        case(rState)
            0:  begin  // idle
                rMDIO = 0; rSDAT = 1;  // send 1
                rBusy = 0;
                rFinish = 0;
                if({rStart_dly, rStart}==2'b01) begin
                    rState = 1;  
                    rMD_RWn = rRWn;
                    rTXData_Reg = rTXData_TempReg;
                    rRXData_TempReg = 0;
                end
            end
            1:  begin   // start
                rMDIO = 0; rBusy = 1; rState = 2; rSDAT = 0;
                end
            2:  begin
                rMDIO = 0; rState = 3; rSDAT = 1;
                end
            3:  begin   // Opcode
                rState = 4;
                rMDIO = 0;
                if(rMD_RWn) rSDAT = 1;
                else rSDAT = 0;
                end
            4:  begin
                rState = 5;
                rMDIO = 0;
                if(rMD_RWn) rSDAT = 0;
                else  rSDAT = 1;
                end
            5:  begin   // PHY_Address
                    rMDIO =0; rSDAT = wPHY_Addr[4]; rState = 6; 
                end
            6:  begin
                    rMDIO =0; rSDAT = wPHY_Addr[3]; rState = 7;
                end
            7:  begin
                    rMDIO =0; rSDAT = wPHY_Addr[2]; rState = 8;
                end
            8:  begin
                    rMDIO =0; rSDAT = wPHY_Addr[1]; rState = 9;
                end
            9:  begin
                    rMDIO =0; rSDAT = wPHY_Addr[0]; rState = 10;
                end
            10:  begin  // Register address
                    rMDIO =0; rSDAT = rREG_Addr[4]; rState = 11;
                end
            11:  begin
                    rMDIO =0; rSDAT = rREG_Addr[3]; rState = 12;
                end
            12:  begin
                    rMDIO =0; rSDAT = rREG_Addr[2]; rState = 13;
                end
            13:  begin
                   rMDIO =0; rSDAT = rREG_Addr[1]; rState = 14;
                end
            14:  begin
                    rMDIO =0; rSDAT = rREG_Addr[0]; rState = 15;
                end
            15:  begin  // Turnaround
                    rMDIO = rMD_RWn; rSDAT = 1; rState = 16;
                end
            16:  begin
                    rMDIO = rMD_RWn; rSDAT = 0; rState = 17;
                end
            17:  begin  // data (16 bit)
                    rState = 18;
                    rMDIO = rMD_RWn;
                    if(rMD_RWn)   rRXData_TempReg[15] = MDIO;
                    else  rSDAT = rTXData_Reg[15];
                end
            18:  begin
                    rState = 19;
                    rMDIO = rMD_RWn;
                    if(rMD_RWn)   rRXData_TempReg[14] = MDIO;
                    else  rSDAT = rTXData_Reg[14];
                end
            19:  begin
                    rState = 20;
                    rMDIO = rMD_RWn;
                    if(rMD_RWn)   rRXData_TempReg[13] = MDIO;
                    else  rSDAT = rTXData_Reg[13];
                end
            20:  begin
                    rState = 21;
                    rMDIO = rMD_RWn;
                    if(rMD_RWn)   rRXData_TempReg[12] = MDIO;
                    else  rSDAT = rTXData_Reg[12];
                end
            21:  begin
                    rState = 22;
                    rMDIO = rMD_RWn;
                    if(rMD_RWn)   rRXData_TempReg[11] = MDIO;
                    else  rSDAT = rTXData_Reg[11];
                end
            22:  begin
                    rState = 23;
                    rMDIO = rMD_RWn;
                    if(rMD_RWn)   rRXData_TempReg[10] = MDIO;
                    else  rSDAT = rTXData_Reg[10];
                end
            23:  begin
                    rState = 24;
                    rMDIO = rMD_RWn;
                    if(rMD_RWn)   rRXData_TempReg[9] = MDIO;
                    else  rSDAT = rTXData_Reg[9];
                end
            24:  begin
                    rState = 25;
                    rMDIO = rMD_RWn;
                    if(rMD_RWn)   rRXData_TempReg[8] = MDIO;
                    else  rSDAT = rTXData_Reg[8];
                end
            25:  begin
                    rState = 26;
                    rMDIO = rMD_RWn;
                    if(rMD_RWn)   rRXData_TempReg[7] = MDIO;
                    else  rSDAT = rTXData_Reg[7];
                end
            26:  begin
                    rState = 27;
                    rMDIO = rMD_RWn;
                    if(rMD_RWn)   rRXData_TempReg[6] = MDIO;
                    else  rSDAT = rTXData_Reg[6];
                end
            27:  begin
                    rState = 28;
                    rMDIO = rMD_RWn;
                    if(rMD_RWn)   rRXData_TempReg[5] = MDIO;
                    else  rSDAT = rTXData_Reg[5];
                end
            28:  begin
                    rState = 29;
                    rMDIO = rMD_RWn;
                    if(rMD_RWn)   rRXData_TempReg[4] = MDIO;
                    else  rSDAT = rTXData_Reg[4];
                end
            29:  begin
                    rState = 30;
                    rMDIO = rMD_RWn;
                    if(rMD_RWn)   rRXData_TempReg[3] = MDIO;
                    else  rSDAT = rTXData_Reg[3];
                end
            30:  begin
                    rState = 31;
                    rMDIO = rMD_RWn;
                    if(rMD_RWn)   rRXData_TempReg[2] = MDIO;
                    else  rSDAT = rTXData_Reg[2];
                end
            31:  begin
                    rState <= 32;
                    rMDIO = rMD_RWn;
                    if(rMD_RWn)   rRXData_TempReg[1] = MDIO;
                    else  rSDAT = rTXData_Reg[1];
                end
            32:  begin
                    rState = 33;
                    rMDIO = rMD_RWn;
                    if(rMD_RWn)   rRXData_TempReg[0] = MDIO;
                    else  rSDAT = rTXData_Reg[0];
                end
            33:  begin // idle
                    rRXData_Reg <= rRXData_TempReg;
                    rState = 0;
                    rMDIO = 0;
                    rFinish = 1;
                    rSDAT = 1;
                end
            default:  begin
                
                end
        endcase
    end
end

endmodule
